package com.devaux.PointInterpolationApp;



// import smile.*;

import java.awt.*;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.lang.Float.parseFloat;
import static java.lang.Integer.parseInt;

public class MappingDataProvider {

    public static final String FILE_CSV = "./file.csv";
    public static final String COMMA_DELIMITER = ";";

    private static MappingDataProvider _INSTANCE;

    private final List<PointInterpolation> datas;
    private  PointInterpolation[] dataArray;
    private static final double maxDistance = 0.01;  // max distance to compute inverse Distance Weighting Interpolation 

    private MappingDataProvider() {
        // Singleton
        this.datas = new ArrayList<PointInterpolation>();
        try {
            loadCSVFile();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public static MappingDataProvider getInstance() {
        if (_INSTANCE == null) {
            _INSTANCE = new MappingDataProvider();
        }
        return _INSTANCE;
    }

    public void loadCSVFile() throws Exception {
        final List<PointInterpolation> records = new ArrayList<>();
        try (BufferedReader br = new BufferedReader(new FileReader(FILE_CSV))) {
            String line;
            while ((line = br.readLine()) != null) {
                final String[] values = line.split(COMMA_DELIMITER);
                final Pattern pattern = Pattern.compile("POINT\\((\\d+\\.\\d+) (\\d+\\.\\d+)\\)");

                final Matcher m = pattern.matcher(values[0]);
                while (m.find()) {
                    final PointInterpolation p = new PointInterpolation(parseFloat(m.group(2)),
                            parseFloat(m.group(1)), parseInt(values[1]));
                    records.add(p);
                }

            }
        }
        datas.addAll(records);
      //  dataArray = datas.toArray();

    }


    public PointInterpolation checkClosePoint(Double x, Double y) {
        PointInterpolation result = null;
        final PointInterpolation toFind = new PointInterpolation(x, y, 0);
        if (toFind instanceof PointInterpolation) {
            final PointInterpolation found = datas.stream()
                    .filter(point -> toFind.equals(point))
                    .findAny()
                    .orElse(null);
            if (found != null) {
                result = found;
            }
        }
        return result;
    }

    // We use Inverse Distance Weighting Interpolation to return interpolated price value
    public PointInterpolation getInterpolation(Double x, Double y, Double d) {

        PointInterpolation result = new PointInterpolation(x, y, 0); // dataArray[0];
        final PointInterpolation inputPoint = new PointInterpolation(x, y, 0);
        double ratioSum = 0;
        double divisorSum = 0;

        // We go through the whole array just once (still a binary search to closer vertices could improve speed)
        // When points are close we use them in the IDW (we define a max distance as a threshold)
        for (int i = 0; i < datas.size(); i++) {
            PointInterpolation currentPoint = datas.get(i);
            double dist = currentPoint.distanceTo(x,y);
            if(dist < d /*maxDistance*/){   
                ratioSum += currentPoint.getInterpolation() / dist;
                divisorSum += 1. / dist;  // We could try to power it to smooth
            }
        }
        
        result.setInterpolation(ratioSum / divisorSum);
        return result;
    }


    
    // Return the nearest neighbours price values to a position
    public List<PointInterpolation> getNearestNeighbours(Double x, Double y, Double d) {

        final PointInterpolation inputPoint = new PointInterpolation(x, y, 0);
        List<PointInterpolation> recordsOut = new ArrayList<>(); // Output of neighbours

        // We go through the whole array just once (still a binary search to closer vertices could improve speed)
        for (int i = 0; i < datas.size(); i++) {
            PointInterpolation currentPoint = datas.get(i);
            double dist = currentPoint.distanceTo(x,y);
            if(dist < d /*maxDistance*/){   
                recordsOut.add(currentPoint);
            }
        }
        
        return recordsOut;

    }

}
